(function($) {
    window.pmSearch = window.pmSearch || {};
    let search = window.pmSearch;

    search.init = function() {
        // init key
        search.getApiKey();
        search.timers = {};
        search.selected = {};

        search.wrapper = $('<div class="pm-tools-search-wrapper hide-form"><div class="search-form"><input type="text" /><ul class="found"></ul></div></div>');
        search.input = search.wrapper.find('input');
        search.found = search.wrapper.find('.found');

        $('#wrapper').append(search.wrapper);

        $('body').bind('keydown', function(event) {
            if (search.wrapper.hasClass('hide-form') &&
                (event.altKey === true && (event.keyCode === 68 || event.keyCode === 206 || event.keyCode === 75))) {
                search.showForm();
                event.preventDefault();
            }
            else if(!search.wrapper.hasClass('hide-form')) {
                if(event.keyCode === 27 || (event.altKey === true && (event.keyCode === 68 || event.keyCode === 75 || event.keyCode === 206))) {
                    search.closeForm();
                    event.preventDefault();
                }
            }
        })
            .bind('click', function (event) {
                if (!search.wrapper.hasClass('hide-form')) {
                    if ($(event.target).closest('.pm-tools-search-wrapper').length === 0) {
                        search.closeForm();
                    }
                }
            });

        search.input.on('keyup', function() {
            let $this = $(this);
            if (search.timers.change) {
                clearTimeout(search.timers.change);
            }

            search.timers.change = setTimeout(function () {
                search.searchValue($this.val());
            }, 100);
        });
        search.wrapper.on('keydown', function (event) {
           let keyCode = event.keyCode;
           if (keyCode === 40 || keyCode === 38) {
               search.navigate(keyCode === 40 ? 'down' : 'up');
               event.preventDefault();
               return false;
           }
           else {
               search.input.focus();
           }
        });
    };

    search.showForm = function() {
        search.input.val('');
        search.found.html('');
        search.wrapper.removeClass('hide-form');
        search.input.focus();
    };
    search.closeForm = function() {
        search.wrapper.addClass('hide-form');
    };

    search.searchValue = function(searchText) {
        if (searchText !== '' && searchText !== search.lastSearch) {
            search.lastSearch = searchText;
            searchText = searchText.replace(' ', '+');
            let key = search.getApiKey();

            if (key) {
                $.ajax({
                    url: "https://pm.i20.biz/issues.json?limit=10&subject=~" + searchText,
                    data: {"": key},
                    dataType: 'json',
                    beforeSend: function(xhr){xhr.setRequestHeader('X-Redmine-API-Key', key);},
                    success: function (data) {
                        if (data && data.total_count) {
                            if (search.currentElement) {
                                search.currentElement = undefined;
                            }

                            search.found.html('');
                            data.issues.forEach(function(element) {
                                let issue = $('<li><a href="https://pm.i20.biz/issues/' + element.id + '">'
                                    + '<div class="found-issue">'
                                    + element.subject
                                    +'</div>'
                                    + '</a></li>'
                                )
                                search.found.append(issue);
                            });
                        }
                    },
                    async: true
                });
            }
            else {
                search.found.append('Введите API ключ в настройках экстеншена.')
            }
        }
        else {
            search.found.html('')
        }
    }

    search.getApiKey = function() {
        if (typeof search.apiKey == 'undefined') {
            chrome.storage.sync.get('apiKey', function(value) {search.apiKey = value.apiKey });
        }
        return search.apiKey;
    }

    search.navigate = function(dest) {
        if (typeof search.currentElement === 'undefined') {
            search.select(0, search.found.find('li:first'));
        }
        let nextDelta = 0;
        let allFound = search.found.find('li');
        let foundCount = allFound.length -1;

        if (dest === 'up') {
            if (search.currentElement.delta === 0) {
                nextDelta = foundCount;
            }
            else {
                nextDelta = search.currentElement.delta -1;
            }
        }
        else if (dest === 'down') {
            if (search.currentElement.delta === foundCount) {
                nextDelta = 0;
            }
            else {
                nextDelta = search.currentElement.delta + 1;
            }
            search.select(nextDelta, $(allFound.eq(nextDelta)));
        }
        search.select(nextDelta, $(allFound.eq(nextDelta)));

    }
    search.select = function(delta, element) {
        if (element.length) {
            search.found.find('li').removeClass('active');
            search.currentElement = {
                element: element,
                delta: delta
            };
            search.currentElement.element.addClass('active').find('a').focus();
        }

    }
    search.init();

})(jQuery);
